// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SceneGen.hpp"

using namespace Dawn;

void SceneGen::generate(
  std::vector<std::string> *out,
  struct Scene *scene,
  std::string tabs
) {
  struct ClassGenInfo classInfo;
  classInfo.clazz = scene->name;
  classInfo.extend = "Scene";
  classInfo.constructorArgs = "DawnGame *game";
  classInfo.extendArgs = "game";

  struct MethodGenInfo methodAssets;
  methodAssets.name = "getRequiredAssets";
  methodAssets.isOverride = true;
  methodAssets.type = "std::vector<Asset*>";
  line(&methodAssets.body, "auto assMan = &this->game->assetManager;", "");
  line(&methodAssets.body, "std::vector<Asset*> assets;", "");

  struct MethodGenInfo methodInit;
  methodInit.name = "stage";
  methodInit.isOverride = true;

  classInfo.includes.push_back("scene/Scene.hpp");


  // Generate
  int32_t assetNumber = 0;
  int32_t childNumber = 0;
  int32_t componentNumber = 0;
  std::map<std::string, std::string> assetMap;

  auto sceneItems = scene->items.begin();
  while(sceneItems != scene->items.end()) {
    SceneItemGenerator::generate(
      assetNumber,
      componentNumber,
      childNumber,
      assetMap,
      classInfo.includes,
      &classInfo.publicProperties,
      &methodInit.body,
      &methodAssets.body,
      "",
      "this",
      &(*sceneItems),
      ""
    );
    ++sceneItems;
  }

  // Seal methods
  line(&methodAssets.body, "return assets;", "");

  // Add in methods
  CodeGen::methodGen(&classInfo.publicCode, methodAssets);
  line(&classInfo.publicCode, "", "");
  CodeGen::methodGen(&classInfo.publicCode, methodInit);

  CodeGen::classGen(out, classInfo);
}